#include <windows.h>
#include <winsock.h>
#include <stdio.h>
#include <string.h> // for strstr()
#include "lglcd.h"
#include "resource.h"
#pragma comment(lib, "wsock32.lib")
#pragma comment(lib, "lgLcd.lib")

#define APP_NAME		"ircMon"
#define WM_SOCKET_NOTIFY	(WM_APP + 1)
#define WMU_UPDATE		(WM_APP + 2)
#define WMU_KEYDOWN		(WM_APP + 3)
#define WMU_CONFIGURE		(WM_APP + 4)

#define SIZE_NAME	512

typedef struct
{
	BOOL		negateLcd;
	unsigned	port, mode;
	char		server[81], nick[31], chan[31], user[31], realName[31];
} appData;

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
DWORD WINAPI buttonCallback(int, DWORD, void *);
DWORD WINAPI configureCallback(int, void *);
BOOL CALLBACK configureProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

void initLcd(HWND, lgLcdConnectContext *, lgLcdDeviceDesc *, lgLcdOpenContext *);
SOCKET initConnection(HWND, appData *);
void updateLcd(HWND, HBITMAP, lgLcdOpenContext *, BOOL);
void processMsg(SOCKET, char *, appData *);
int printLcdString(HDC, int, int, int, char *, int);
void convertBitmap(BYTE *, BYTE *, int);
void initFont(LOGFONT *);
int initData(HWND hwnd, appData *);
BOOL lcdNeg(BOOL);

FILE *debug;

// Crappy rotating list data structure
#define SIZE_MESSAGE	512
#define SIZE_LINES	15

char	RLIST[SIZE_LINES][SIZE_MESSAGE];
int	RLIST_POS;
int	RLIST_COUNT;

void addString(char *str)
{
	if (RLIST_COUNT < SIZE_LINES)
	{
		lstrcpy(&RLIST[RLIST_POS + RLIST_COUNT++][0], str);
		return;
	}

	lstrcpy(&RLIST[RLIST_POS++][0], str);
	if (RLIST_POS == SIZE_LINES)
		RLIST_POS = 0;
	return;
}

char *getString(int num)
{
	if (RLIST_POS + num >= SIZE_LINES )
	{
		return &RLIST[RLIST_POS + num - SIZE_LINES][0];
	}
	return &RLIST[RLIST_POS + num][0];
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine, int iCmdShow)
{
	HWND		hwnd;
	MSG		msg;
	WNDCLASS	wndclass;
	const char	szAppName[] = APP_NAME;

	wndclass.style		= CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc	= WndProc;
	wndclass.cbClsExtra	= 0;
	wndclass.cbWndExtra	= 0;
	wndclass.hInstance	= hInstance;
	wndclass.hIcon		= LoadIcon(hInstance, szAppName);
	wndclass.hCursor	= LoadCursor(NULL, IDC_ARROW);
	wndclass.hbrBackground	= (HBRUSH) GetStockObject(WHITE_BRUSH);
	wndclass.lpszMenuName	= NULL;
	wndclass.lpszClassName	= szAppName;

	if (!RegisterClass(&wndclass))
	{
		MessageBox(NULL, "Unable to register window class.", szAppName, MB_ICONERROR);
		return 0;
	}

	hwnd = CreateWindow(	szAppName,			// window class name
				APP_NAME,			// window caption
				WS_OVERLAPPEDWINDOW,		// window style
				CW_USEDEFAULT,			// initial x position
				CW_USEDEFAULT,			// initial y position
				CW_USEDEFAULT,			// initial x size
				CW_USEDEFAULT,			// initial y size
				NULL,				// parent window handle
				NULL,				// window menu handle
				hInstance,			// program instance handle
				NULL);				// creation parameters

	ShowWindow(hwnd, iCmdShow);
	UpdateWindow(hwnd);
	while (GetMessage(&msg, NULL, 0, 0) > 0)
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return msg.wParam;
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static lgLcdConnectContext	connectContext;
	static lgLcdDeviceDesc		deviceDescription;
	static lgLcdOpenContext		openContext;
	static BOOL			lcdOpen = FALSE;
	static WSADATA			WSAData;
	static SOCKET			sock;
	static HFONT			hFont, hOldFont;
	static HBITMAP			hLcdBitmap;
	static HDC			hdcMem;
	static appData			data;
	static				scrollDist;

	switch (message)
	{
	case WM_CREATE:
		{
			char		buffer[80];
			LOGFONT		lf;

			debug = fopen("debug.txt", "w");
			initLcd(hwnd, &connectContext, &deviceDescription, &openContext);
			lcdOpen = TRUE;
			hLcdBitmap = CreateBitmap(deviceDescription.Width, deviceDescription.Height, deviceDescription.Bpp, 0, NULL);
			hdcMem = CreateCompatibleDC(NULL);
			SelectObject(hdcMem, hLcdBitmap);
		
			SetTextColor(hdcMem, RGB(255,255,255));
			SetBkMode(hdcMem, TRANSPARENT);
			SetViewportOrgEx(hdcMem, 2, deviceDescription.Height, NULL);
			SetTextAlign(hdcMem, TA_BOTTOM);
		
			initFont(&lf);
			if (initData(hwnd, &data))
			{
				SendMessage(hwnd, WMU_CONFIGURE, 0, 0);
				if (initData(hwnd, &data))
				{
					SendMessage(hwnd, WM_DESTROY, 0, 0);
					return 0;
				}
			}
			hFont = CreateFontIndirect(&lf);
			hOldFont = SelectObject(hdcMem, hFont);

			wsprintf(buffer, "Connecting to %s", data.server);
			addString(buffer);
			PostMessage(hwnd, WMU_UPDATE, 0, 0);

			if ( WSAStartup(MAKEWORD(2,0), &WSAData) )
			{
				MessageBox(NULL, "Unable to start winsock.", APP_NAME, MB_ICONERROR);
				SendMessage(hwnd, WM_DESTROY, 0, 0);
				return 0;
			}

			sock = initConnection(hwnd, &data);
		}
		return 0;

	case WMU_UPDATE:
		{
			int		i, length, extraLines = 0, ret;
			static		iLast = -1;
			char str[512];

			for(i = 0; i < RLIST_COUNT; i++)
			{
				lstrcpy(str, getString(i));
				length = lstrlen(str);
				ret = printLcdString(hdcMem, deviceDescription.Width-5, 0, -RLIST_COUNT + 1 + i + scrollDist + extraLines, str, length);
				extraLines += ret;
				if (i+1 == RLIST_COUNT && i != iLast)
				{
					iLast = i;
					scrollDist -= ret; //must be done once wrapped string added, not everytime screen updated
				}
			}
			updateLcd(hwnd, hLcdBitmap, &openContext, data.negateLcd);
		}
		return 0;

	case WMU_CONFIGURE:
		DialogBox((HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE), "config", hwnd, configureProc);
		return 0;

	case WM_PAINT:
		{
			HDC		hdc;
			PAINTSTRUCT	ps;
			TEXTMETRIC	tm;
			RECT		rect;
			char		path[MAX_PATH];
			int		i;

			GetCurrentDirectory(MAX_PATH, path);
			lstrcat(path, "\\ircMon.ini");

			GetClientRect(hwnd, &rect);
			hdc = BeginPaint(hwnd, &ps);
			GetTextMetrics(hdc, &tm);
			TextOut(hdc, 200, 0, path, lstrlen(path));
			for(i = 0; i < RLIST_COUNT; i++)
				TextOut(hdc, 0, tm.tmHeight * i, getString(i), lstrlen(getString(i)));
			EndPaint(hwnd, &ps);
		}
		return 0;

	case WM_SOCKET_NOTIFY:
		{
			char	buffer[SIZE_MESSAGE];
			char	serverMsg[SIZE_MESSAGE + 1];
			int	i, j, iSize;

			if (WSAGETSELECTERROR(lParam) == WSAENETDOWN)
				return 0;

			switch(WSAGETSELECTEVENT(lParam))
			{
			case FD_CONNECT:
				{
					switch(WSAGETSELECTERROR(lParam))
					{
					case WSAECONNREFUSED:
						return 0;
					case WSAETIMEDOUT:
						return 0;
					}

					wsprintf(buffer, "NICK %s\r\n", data.nick);
					send( sock, buffer, lstrlen(buffer), 0);
					wsprintf(buffer, "USER %s %u * :%s\r\n", data.user, data.mode, data.realName);
					send( sock, buffer, lstrlen(buffer), 0);
				}
				break;	
			case FD_READ:
				iSize = recv(sock, (char *) &buffer, SIZE_MESSAGE, MSG_PEEK);
				for(i = 0, j = 0; i + 1 < iSize; i++, j++)
				{
					if (buffer[i] == '\r' && buffer[i + 1] == '\n')
					{
						i++;
						recv(sock, serverMsg, j + 1, 0);
						serverMsg[j + 1] = '\0';
						processMsg(sock, serverMsg, &data);
						j = 0;
					}
				}
				PostMessage(hwnd, WMU_UPDATE, 0, 0);
				InvalidateRect(hwnd, NULL, TRUE);
				break;
			case FD_CLOSE:
				closesocket(sock);
				break;
				
			};
		}
		return 0;

	case WMU_KEYDOWN:
	case WM_KEYDOWN:
		switch (wParam)
		{
		case VK_UP:
			scrollDist++;
			SendMessage(hwnd, WMU_UPDATE, 0, 0);

			break;
			
		case VK_DOWN:
			scrollDist--;
			SendMessage(hwnd, WMU_UPDATE, 0, 0);
			break;
		}
		return 0;

	case WM_DESTROY:
		{
			fclose(debug);

			hFont = SelectObject(hdcMem, hOldFont);
			DeleteDC(hdcMem);
			DeleteObject(hLcdBitmap);
			DeleteObject(hFont);

			if (lcdOpen)
			{
				// Errors dont matter, closing anyhow.
				lgLcdClose(openContext.device);
				lgLcdDisconnect(connectContext.connection);
			}
			lgLcdDeInit();

			if (sock != INVALID_SOCKET)
				closesocket(sock);
			WSACleanup();
			PostQuitMessage(0);
		}
		return 0;
	}
	return DefWindowProc(hwnd, message, wParam, lParam);
}

void processMsg(SOCKET sock, char *buffer, appData *data)
{
	char		prefix[SIZE_MESSAGE], command[SIZE_MESSAGE], buff[512];
	BOOL		bPrefix = FALSE;
	char		*temp;
	int		i;

	fprintf(debug, "~%s~~\n", buffer);
	fflush(debug);

	if (buffer[0] == ':')
	{
		bPrefix = TRUE;
		for (i = 1;; i++)
		{
			if (buffer[i] == '\0')
				return;
			if (buffer[i] == ':')
				break;
			prefix[i - 1] = buffer[i];
		}
		prefix[i] = '\0';
		buffer += i + 1;
	}

	if (bPrefix)
	{
		sscanf(prefix, "%*s %s", &command);
		if ( !strcmp(command, "001") )
		{
			char join[512];

			wsprintf(join, "JOIN %s\r\n", data->chan);
			addString("Connected");
			send( sock, join, lstrlen(join), 0);
			return;
		}

		if ( !strcmp(command, "447") )
		{
			wsprintf(buff, "*** %s", buffer);
			addString(buff);
			return;
		}

		if ( !strcmp(command, "PRIVMSG") )
		{
			char buff[512];
			char user[SIZE_NAME];
			char *pointer;

			sscanf(prefix, "%s", user);
			pointer = strstr(user, "!");
			if (pointer)
			{
				*pointer ='\0';
				wsprintf(buff, "%s: %s", user, buffer);
				addString(buff);
			}
			return;
		}

		if ( !strcmp(command, "JOIN") )
		{
			char user[SIZE_MESSAGE], chan[SIZE_MESSAGE];
			char *pointer;

			sscanf(prefix, "*** %s %*s %s", user, chan);
			pointer = strstr(user, "!");
			if (pointer)
			{
				*pointer = '\0';
				wsprintf(buff, "%s joined %s", user, chan);
				addString(buff);
			}
			return;
		}

		if ( !strcmp(command, "PART") )
		{
			char user[SIZE_NAME], chan[SIZE_MESSAGE];
			char *pointer;

			sscanf(prefix, "%s %*s %s", user, chan);
			pointer = strstr(user, "!");
			if (pointer)
			{
				*pointer = '\0';
				wsprintf(buff, "*** %s left %s", user, chan);
				addString(buff);
			}
			return;
		}

		if ( !strcmp(command, "QUIT") )
		{
			char user[SIZE_NAME];
			char quitmsg[SIZE_MESSAGE];
			char *pointer;

			sscanf(prefix, "%s %*s %s", user, quitmsg);
			pointer = strstr(user, "!");
			if (pointer)
			{
				*pointer = '\0';
				wsprintf(buff, "*** %s quit \"%s\"", user, quitmsg);
				addString(buff);
			}
			return;
		}

		if ( !strcmp(command, "KICK") )
		{
			char op[SIZE_NAME], user[SIZE_NAME];
			char *pointer;

			sscanf(prefix, "%s %*s %*s %s", &op, &user);
			pointer = strstr(op, "!");
			if (pointer)
			{
				*pointer = '\0';
				wsprintf(buff, "%s was kicked by %s for %s", user, op, buffer);
				addString(buff);
			}

			if ( !strcmp(user, data->nick) )
			{
				char join[512];

				wsprintf(join, "JOIN %s\r\n", data->chan);
				send( sock, join, lstrlen(join), 0);
			}
			return;
		}
	}

	if ( strstr(buffer, "PING :") )
	{
		buffer[1] = 'O';
		lstrcat(buffer, "\r\n");
		send( sock, buffer, lstrlen(buffer), 0);
		return;
	}
}

SOCKET initConnection(HWND hwnd, appData *data)
{
	SOCKADDR_IN		sa;
	LPHOSTENT 		host;
	SOCKET			sock;
	int			iError;

	sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

	if ( sock == INVALID_SOCKET )
	{
		return INVALID_SOCKET;
	}

	if ( WSAAsyncSelect(sock, hwnd, WM_SOCKET_NOTIFY, FD_CONNECT | FD_READ | FD_CLOSE) == SOCKET_ERROR )
	{
		closesocket(sock);
		return INVALID_SOCKET;
	}

	sa.sin_family		= AF_INET;
	sa.sin_port		= htons(data->port);
	host			= gethostbyname(data->server);

	if(host)
		sa.sin_addr = *((LPIN_ADDR)*host->h_addr_list);
	else
		sa.sin_addr.S_un.S_addr	= inet_addr(data->server);

	connect(sock, (SOCKADDR *) &sa, sizeof(SOCKADDR_IN));
	iError = WSAGetLastError();

	if ( iError )
	{
		if ( iError != WSAEWOULDBLOCK )
		{
			closesocket(sock);
			return INVALID_SOCKET;
		}
	}
	return sock;
}

void initLcd(HWND hwnd, lgLcdConnectContext *connectContext, lgLcdDeviceDesc *deviceDescription, lgLcdOpenContext *openContext)
{
	static HWND	hwndstat;	// memory must exist when called after this function terminates
	lgLcdSoftbuttonsChangedContext	buttonContext;
	DWORD				ret;

	hwndstat = hwnd;

	// setup connect structure
	ZeroMemory(connectContext, sizeof(lgLcdConnectContext));
	connectContext->appFriendlyName			= "Logitech G-series IRC Monitor";
	connectContext->isAutostartable			= TRUE;
	connectContext->isPersistent			= TRUE;
	connectContext->connection			= LGLCD_INVALID_CONNECTION;
	connectContext->onConfigure.configCallback	= configureCallback;
	connectContext->onConfigure.configContext	= &hwndstat;

	ret = lgLcdInit();
	//checkError(hwnd, ret, "lgLcdInit()");
	ret = lgLcdConnect(connectContext);
	//checkError(hwnd, ret, "lgLcdConnect()");
	ret = lgLcdEnumerate(connectContext->connection, 0, deviceDescription);
	//checkError(hwnd, ret, "lgLcdEnumerate()");

	// Init openContext with some data from connectContext
	ZeroMemory(openContext, sizeof(lgLcdOpenContext));
	openContext->connection		= connectContext->connection;
	openContext->index		= 0;
	openContext->device		= 0;
	openContext->onSoftbuttonsChanged.softbuttonsChangedCallback	= buttonCallback;
	openContext->onSoftbuttonsChanged.softbuttonsChangedContext	= &hwndstat;

	ret = lgLcdOpen(openContext);
	//checkError(hwnd, ret, "lgLcdOpen()");
}

DWORD WINAPI buttonCallback(int device, DWORD dwButtons, void *hwnd)
{
	switch (dwButtons)
	{
	case LGLCDBUTTON_BUTTON0:
		PostMessage(*((HWND *)hwnd), WMU_KEYDOWN, VK_UP, 0);
		PostMessage(*((HWND *)hwnd), WMU_UPDATE, 0, 0);
		break;
	case LGLCDBUTTON_BUTTON1:
		PostMessage(*((HWND *)hwnd), WMU_KEYDOWN, VK_DOWN, 0);
		PostMessage(*((HWND *)hwnd), WMU_UPDATE, 0, 0);
		break;
	case LGLCDBUTTON_BUTTON2:
		break;
	case LGLCDBUTTON_BUTTON3:
		break;
	}
	return 0;
}

DWORD WINAPI configureCallback(int device, void *hwnd)
{
	PostMessage(*((HWND *)hwnd), WMU_CONFIGURE, 0, 0);
	return 0;
}

BOOL CALLBACK configureProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	static char	path[MAX_PATH];

	switch(message)
	{
	case WM_INITDIALOG:
		{
			char	buffer[80];
			BOOL	negate;

			GetCurrentDirectory(MAX_PATH, path);
			lstrcat(path, "\\ircMon.ini");

			//get existing values if any
			GetPrivateProfileString("ircMon", "server", "127.0.0.1", buffer, 80, path);
			SetWindowText(GetDlgItem(hDlg, IDC_SERVER), buffer);

			GetPrivateProfileString("ircMon", "port", "6667", buffer, 80, path);
			SetWindowText(GetDlgItem(hDlg, IDC_PORT), buffer);

			GetPrivateProfileString("ircMon", "nick", NULL, buffer, 80, path);
			SetWindowText(GetDlgItem(hDlg, IDC_NICK), buffer);

			GetPrivateProfileString("ircMon", "channel", NULL, buffer, 80, path);
			SetWindowText(GetDlgItem(hDlg, IDC_CHANNEL), buffer);

			GetPrivateProfileString("ircMon", "username", NULL, buffer, 80, path);
			SetWindowText(GetDlgItem(hDlg, IDC_USERNAME), buffer);

			GetPrivateProfileString("ircMon", "realname", NULL, buffer, 80, path);
			SetWindowText(GetDlgItem(hDlg, IDC_REALNAME), buffer);

			GetPrivateProfileString("ircMon", "mode", NULL, buffer, 80, path);
			SetWindowText(GetDlgItem(hDlg, IDC_MODE), buffer);

			GetPrivateProfileString("ircMon", "negate", NULL, buffer, 80, path);
			sscanf(buffer, "%d", &negate);
			SendMessage(GetDlgItem(hDlg, IDC_NEG), BM_SETCHECK, negate, 0);
		}
		return TRUE;
	case WM_COMMAND:
		switch( LOWORD(wParam) )
		{
		case IDOK:
			{
				char	buffer[80];
				BOOL	negate;

				//write values entered
				GetWindowText(GetDlgItem(hDlg, IDC_SERVER), buffer, 80);
				WritePrivateProfileString("ircMon", "server", buffer, path);

				GetWindowText(GetDlgItem(hDlg, IDC_PORT), buffer, 80);
				WritePrivateProfileString("ircMon", "port", buffer, path);

				GetWindowText(GetDlgItem(hDlg, IDC_NICK), buffer, 80);
				WritePrivateProfileString("ircMon", "nick", buffer, path);

				GetWindowText(GetDlgItem(hDlg, IDC_CHANNEL), buffer, 80);
				WritePrivateProfileString("ircMon", "channel", buffer, path);

				GetWindowText(GetDlgItem(hDlg, IDC_USERNAME), buffer, 80);
				WritePrivateProfileString("ircMon", "username", buffer, path);

				GetWindowText(GetDlgItem(hDlg, IDC_REALNAME), buffer, 80);
				WritePrivateProfileString("ircMon", "realname", buffer, path);

				GetWindowText(GetDlgItem(hDlg, IDC_MODE), buffer, 80);
				WritePrivateProfileString("ircMon", "mode", buffer, path);

				negate = (int)SendMessage(GetDlgItem(hDlg, IDC_NEG), BM_GETCHECK, 0, 0);
				WritePrivateProfileString("ircMon", "negate", buffer, path);

				EndDialog(hDlg, 0);
				return TRUE;
			}
		case IDCANCEL:
			EndDialog(hDlg, 0);
			return TRUE;
		}
		break;
	}
	return FALSE;
}

void updateLcd(HWND hwnd, HBITMAP hBitmap, lgLcdOpenContext *openContext, BOOL negate)
{
	lgLcdBitmap160x43x1	bitmap;
	lgLcdBitmapHeader	header;
	BYTE			winbitmap[860];
	DWORD			ret;

	// set up structures
	header.Format = LGLCD_BMP_FORMAT_160x43x1;
	bitmap.hdr = header;

	GetBitmapBits(hBitmap, 860, &winbitmap);
	if (negate)
	{
		int i;
		for (i = 0; i < 860; i++)
			winbitmap[i] = ~winbitmap[i];
	}
	convertBitmap(winbitmap, bitmap.pixels, 860);

	//update lcd display
	ret = lgLcdUpdateBitmap(openContext->device, &(bitmap.hdr), LGLCD_ASYNC_UPDATE(LGLCD_PRIORITY_NORMAL));
	//checkError(hwnd, ret, "lgLcdUpdateBitmap()");
	ZeroMemory(&winbitmap, 860);
	SetBitmapBits(hBitmap, 860, &winbitmap);
}

void convertBitmap(BYTE *bitAligned, BYTE *byteAligned, int size)
{
	int i, j;

	for (i = 0, j = 0; i < size; i++)
	{
		byteAligned[j++] = (bitAligned[i] & 0x80) ? 0xFF : 0x00;
		byteAligned[j++] = (bitAligned[i] & 0x40) ? 0xFF : 0x00;
		byteAligned[j++] = (bitAligned[i] & 0x20) ? 0xFF : 0x00;
		byteAligned[j++] = (bitAligned[i] & 0x10) ? 0xFF : 0x00;
		byteAligned[j++] = (bitAligned[i] & 0x08) ? 0xFF : 0x00;
		byteAligned[j++] = (bitAligned[i] & 0x04) ? 0xFF : 0x00;
		byteAligned[j++] = (bitAligned[i] & 0x02) ? 0xFF : 0x00;
		byteAligned[j++] = (bitAligned[i] & 0x01) ? 0xFF : 0x00;
	}
}

void initFont(LOGFONT *lf)
{
	lf->lfHeight			= 12;
	lf->lfWidth			= 5;
	lf->lfEscapement		= 0;
	lf->lfOrientation		= 0;
	lf->lfWeight			= 400;
	lf->lfItalic			= 0;
	lf->lfUnderline			= 0;
	lf->lfStrikeOut			= 0;
	lf->lfCharSet			= ANSI_CHARSET;
	lf->lfOutPrecision		= OUT_DEFAULT_PRECIS;
	lf->lfClipPrecision		= CLIP_DEFAULT_PRECIS;
	lf->lfQuality			= NONANTIALIASED_QUALITY;
	lf->lfPitchAndFamily		= DEFAULT_PITCH | FF_DONTCARE;
//	lstrcpy(lf->lfFaceName, "Courier");
}

int initData(HWND hwnd, appData *data)
{
	char	path[MAX_PATH], buffer[81];

	GetCurrentDirectory(MAX_PATH, path);
	lstrcat(path, "\\ircMon.ini");

	GetPrivateProfileString("ircMon", "server", "127.0.0.1", data->server, 31, path);
	if (data->server[0] == '\0')
		return -1;

	GetPrivateProfileString("ircMon", "port", "6667", buffer, 81, path);
	sscanf(buffer, "%d", &data->port);

	GetPrivateProfileString("ircMon", "nick", NULL, data->nick, 31, path);
	if (data->nick[0] == '\0')
		return -1;

	GetPrivateProfileString("ircMon", "channel", NULL, data->chan, 31, path);
	if (data->chan[0] == '\0')
		return -1;

	GetPrivateProfileString("ircMon", "username", NULL, data->user, 31, path);
	if (data->user[0] == '\0')
		return -1;

	GetPrivateProfileString("ircMon", "realname", NULL, data->realName, 31, path);
	if (data->realName[0] == '\0')
		return -1;

	GetPrivateProfileString("ircMon", "mode", NULL, buffer, 81, path);
	sscanf(buffer, "%d", &data->mode);

	GetPrivateProfileString("ircMon", "negate", NULL, buffer, 81, path);
	sscanf(buffer, "%d", &data->negateLcd);
	return 0;
}

int printLcdString(HDC hdc, int lcdWidth, int x, int y, char *str, int length)
{
	TEXTMETRIC	tm;
	SIZE		size;
	int		i, lastSpace = -1, start = 0, extraLines = 0;

	GetTextMetrics(hdc, &tm);
	GetTextExtentPoint32(hdc, str, length, &size);
	if (size.cx < lcdWidth)
	{
		TextOut(hdc, x, tm.tmAscent * y, str, length);
	} else {
		// word wrap code
		for(i = 0; i < length; i++)
		{

			GetTextExtentPoint32(hdc, &str[start], i - start, &size);
			if (size.cx > lcdWidth)	// too big, print last
			{
				if (lastSpace != -1)		// space was found, break there
				{
					TextOut(hdc, x, tm.tmAscent * y++, &str[start], lastSpace - start);
					start = lastSpace + 1;		// move up position
					lastSpace = -1;			// reset space flag
					extraLines++;
					continue;
				} else {
					// large single word
					TextOut(hdc, x, tm.tmAscent * y++, &str[start], i - start );
					start = i;
					extraLines++;
					continue;
				}
			}
			if (str[i] == ' ')
			{
				lastSpace = i;
			}
		}
		TextOut(hdc, x, tm.tmAscent * y, &str[start], length - start );
	}
	return	extraLines;
}